package xhttp3.internal.framed;

import java.io.IOException;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.Collections;
import java.util.Iterator;
import java.util.LinkedHashMap;
import java.util.List;
import java.util.Map;
import xio.Buffer;
import xio.BufferedSource;
import xio.ByteString;
import xio.Okio;
import xio.Source;

final class Hpack {
   private static final int PREFIX_4_BITS = 15;
   private static final int PREFIX_5_BITS = 31;
   private static final int PREFIX_6_BITS = 63;
   private static final int PREFIX_7_BITS = 127;
   private static final Header[] STATIC_HEADER_TABLE;
   private static final Map<ByteString, Integer> NAME_TO_FIRST_INDEX;

   private Hpack() {
   }

   private static Map<ByteString, Integer> nameToFirstIndex() {
      LinkedHashMap var0;
      var0 = new LinkedHashMap.<init>(STATIC_HEADER_TABLE.length);

      Header[] var2;
      for(int var1 = 0; var1 < (var2 = STATIC_HEADER_TABLE).length; ++var1) {
         if (!var0.containsKey(var2[var1].name)) {
            ByteString var3 = var2[var1].name;
            var0.put(var3, var1);
         }
      }

      return Collections.unmodifiableMap(var0);
   }

   private static ByteString checkLowercase(ByteString name) {
      int var1 = 0;

      for(int var2 = name.size(); var1 < var2; ++var1) {
         byte var3;
         if ((var3 = name.getByte(var1)) >= 65 && var3 <= 90) {
            throw new IOException("PROTOCOL_ERROR response malformed: mixed case name: " + name.utf8());
         }
      }

      return name;
   }

   static {
      Header[] var10000 = new Header[61];
      Header var0;
      var0 = new Header.<init>(Header.TARGET_AUTHORITY, "");
      var10000[0] = var0;
      ByteString var1;
      var0 = new Header.<init>(var1 = Header.TARGET_METHOD, "GET");
      var10000[1] = var0;
      var0 = new Header.<init>(var1, "POST");
      var10000[2] = var0;
      var0 = new Header.<init>(var1 = Header.TARGET_PATH, "/");
      var10000[3] = var0;
      var0 = new Header.<init>(var1, "/index.html");
      var10000[4] = var0;
      var0 = new Header.<init>(var1 = Header.TARGET_SCHEME, "http");
      var10000[5] = var0;
      var0 = new Header.<init>(var1, "https");
      var10000[6] = var0;
      var0 = new Header.<init>(var1 = Header.RESPONSE_STATUS, "200");
      var10000[7] = var0;
      var0 = new Header.<init>(var1, "204");
      var10000[8] = var0;
      var0 = new Header.<init>(var1, "206");
      var10000[9] = var0;
      var0 = new Header.<init>(var1, "304");
      var10000[10] = var0;
      var0 = new Header.<init>(var1, "400");
      var10000[11] = var0;
      var0 = new Header.<init>(var1, "404");
      var10000[12] = var0;
      var0 = new Header.<init>(var1, "500");
      var10000[13] = var0;
      var0 = new Header.<init>("accept-charset", "");
      var10000[14] = var0;
      var0 = new Header.<init>("accept-encoding", "gzip, deflate");
      var10000[15] = var0;
      var0 = new Header.<init>("accept-language", "");
      var10000[16] = var0;
      var0 = new Header.<init>("accept-ranges", "");
      var10000[17] = var0;
      var0 = new Header.<init>("accept", "");
      var10000[18] = var0;
      var0 = new Header.<init>("access-control-allow-origin", "");
      var10000[19] = var0;
      var0 = new Header.<init>("age", "");
      var10000[20] = var0;
      var0 = new Header.<init>("allow", "");
      var10000[21] = var0;
      var0 = new Header.<init>("authorization", "");
      var10000[22] = var0;
      var0 = new Header.<init>("cache-control", "");
      var10000[23] = var0;
      var0 = new Header.<init>("content-disposition", "");
      var10000[24] = var0;
      var0 = new Header.<init>("content-encoding", "");
      var10000[25] = var0;
      var0 = new Header.<init>("content-language", "");
      var10000[26] = var0;
      var0 = new Header.<init>("content-length", "");
      var10000[27] = var0;
      var0 = new Header.<init>("content-location", "");
      var10000[28] = var0;
      var0 = new Header.<init>("content-range", "");
      var10000[29] = var0;
      var0 = new Header.<init>("content-type", "");
      var10000[30] = var0;
      var0 = new Header.<init>("cookie", "");
      var10000[31] = var0;
      var0 = new Header.<init>("date", "");
      var10000[32] = var0;
      var0 = new Header.<init>("etag", "");
      var10000[33] = var0;
      var0 = new Header.<init>("expect", "");
      var10000[34] = var0;
      var0 = new Header.<init>("expires", "");
      var10000[35] = var0;
      var0 = new Header.<init>("from", "");
      var10000[36] = var0;
      var0 = new Header.<init>("host", "");
      var10000[37] = var0;
      var0 = new Header.<init>("if-match", "");
      var10000[38] = var0;
      var0 = new Header.<init>("if-modified-since", "");
      var10000[39] = var0;
      var0 = new Header.<init>("if-none-match", "");
      var10000[40] = var0;
      var0 = new Header.<init>("if-range", "");
      var10000[41] = var0;
      var0 = new Header.<init>("if-unmodified-since", "");
      var10000[42] = var0;
      var0 = new Header.<init>("last-modified", "");
      var10000[43] = var0;
      var0 = new Header.<init>("link", "");
      var10000[44] = var0;
      var0 = new Header.<init>("location", "");
      var10000[45] = var0;
      var0 = new Header.<init>("max-forwards", "");
      var10000[46] = var0;
      var0 = new Header.<init>("proxy-authenticate", "");
      var10000[47] = var0;
      var0 = new Header.<init>("proxy-authorization", "");
      var10000[48] = var0;
      var0 = new Header.<init>("range", "");
      var10000[49] = var0;
      var0 = new Header.<init>("referer", "");
      var10000[50] = var0;
      var0 = new Header.<init>("refresh", "");
      var10000[51] = var0;
      var0 = new Header.<init>("retry-after", "");
      var10000[52] = var0;
      var0 = new Header.<init>("server", "");
      var10000[53] = var0;
      var0 = new Header.<init>("set-cookie", "");
      var10000[54] = var0;
      var0 = new Header.<init>("strict-transport-security", "");
      var10000[55] = var0;
      var0 = new Header.<init>("transfer-encoding", "");
      var10000[56] = var0;
      var0 = new Header.<init>("user-agent", "");
      var10000[57] = var0;
      var0 = new Header.<init>("vary", "");
      var10000[58] = var0;
      var0 = new Header.<init>("via", "");
      var10000[59] = var0;
      var0 = new Header.<init>("www-authenticate", "");
      var10000[60] = var0;
      STATIC_HEADER_TABLE = var10000;
      NAME_TO_FIRST_INDEX = nameToFirstIndex();
   }

   static final class Writer {
      private static final byte SEPARATED_TOKEN = 58;
      private static final int SETTINGS_HEADER_TABLE_SIZE = 4096;
      private static final int SETTINGS_HEADER_TABLE_SIZE_LIMIT = 16384;
      private final Buffer out;
      private final Map<ByteString, Integer> headerStringToDynamicIndex;
      private int smallestHeaderTableSizeSetting;
      private boolean emitDynamicTableSizeUpdate;
      int headerTableSizeSetting;
      int maxDynamicTableByteCount;
      Header[] dynamicTable;
      int nextHeaderIndex;
      int headerCount;
      int dynamicTableByteCount;

      Writer(Buffer out) {
         this(4096, out);
      }

      Writer(int headerTableSizeSetting, Buffer out) {
         Writer var10000 = this;
         Writer var10002 = this;
         Writer var10004 = this;
         Writer var10006 = this;
         Writer var10007 = this;
         Writer var10008 = this;
         super();
         this.headerStringToDynamicIndex = new LinkedHashMap();
         this.smallestHeaderTableSizeSetting = Integer.MAX_VALUE;
         Header[] var3;
         this.dynamicTable = var3 = new Header[8];
         var10008.nextHeaderIndex = var3.length - 1;
         var10007.headerCount = 0;
         var10006.dynamicTableByteCount = 0;
         var10004.headerTableSizeSetting = headerTableSizeSetting;
         var10002.maxDynamicTableByteCount = headerTableSizeSetting;
         var10000.out = out;
      }

      private void clearDynamicTable() {
         Arrays.fill(this.dynamicTable, (Object)null);
         this.headerStringToDynamicIndex.clear();
         this.nextHeaderIndex = this.dynamicTable.length - 1;
         this.headerCount = 0;
         this.dynamicTableByteCount = 0;
      }

      private int evictToRecoverBytes(int bytesToRecover) {
         int var2 = 0;
         if (bytesToRecover > 0) {
            int var3;
            int var4;
            for(var3 = this.dynamicTable.length - 1; var3 >= (var4 = this.nextHeaderIndex) && bytesToRecover > 0; bytesToRecover = var4) {
               Header[] bytesToRecover1;
               var4 = bytesToRecover - (bytesToRecover1 = this.dynamicTable)[var3].hpackSize;
               this.dynamicTableByteCount -= bytesToRecover1[var3].hpackSize;
               int var10001 = this.headerCount--;
               ++var2;
               --var3;
            }

            Header[] var7 = this.dynamicTable;
            bytesToRecover = var4 + 1;
            var3 = var4 + 1 + var2;
            var4 = this.headerCount;
            System.arraycopy(var7, bytesToRecover, var7, var3, var4);
            Iterator bytesToRecover2 = this.headerStringToDynamicIndex.entrySet().iterator();

            while(bytesToRecover2.hasNext()) {
               Map.Entry var10000 = (Map.Entry)bytesToRecover2.next();
               var10000.setValue((Integer)var10000.getValue() + var2);
            }

            this.nextHeaderIndex += var2;
         }

         return var2;
      }

      private void insertIntoDynamicTable(Header entry) {
         int var2;
         int var3;
         if ((var2 = entry.hpackSize) > (var3 = this.maxDynamicTableByteCount)) {
            this.clearDynamicTable();
         } else {
            this.evictToRecoverBytes(this.dynamicTableByteCount + var2 - var3);
            Header[] var7;
            if (this.headerCount + 1 > (var7 = this.dynamicTable).length) {
               Header[] var10001 = var7;
               Header[] var10002 = var7;
               Header[] var10003 = var7;
               var7 = new Header[var7.length * 2];
               int var4 = var10003.length;
               int var5 = var10002.length;
               System.arraycopy(var10001, 0, var7, var4, var5);
               Iterator var8 = this.headerStringToDynamicIndex.entrySet().iterator();

               while(var8.hasNext()) {
                  Map.Entry var10000 = (Map.Entry)var8.next();
                  var10000.setValue((Integer)var10000.getValue() + this.dynamicTable.length);
               }

               this.nextHeaderIndex = this.dynamicTable.length - 1;
               this.dynamicTable = var7;
            }

            Writer var9 = this;
            Writer var10 = this;
            Writer var11 = this;
            Writer var12 = this;
            this.nextHeaderIndex = (var3 = this.nextHeaderIndex) - 1;
            this.dynamicTable[var3] = entry;
            Map var10004 = this.headerStringToDynamicIndex;
            ByteString var6 = this.getHeaderString(entry);
            var10004.put(var6, var3);
            var11.headerCount = var12.headerCount + 1;
            var9.dynamicTableByteCount = var10.dynamicTableByteCount + var2;
         }
      }

      private void adjustDynamicTableByteCount() {
         int var1;
         int var2;
         if ((var1 = this.maxDynamicTableByteCount) < (var2 = this.dynamicTableByteCount)) {
            if (var1 == 0) {
               this.clearDynamicTable();
            } else {
               this.evictToRecoverBytes(var2 - var1);
            }
         }

      }

      ByteString getHeaderString(Header entry) {
         byte[] var3;
         byte[] var10000 = var3 = new byte[entry.name.size() + 1 + entry.value.size()];
         byte[] var10004 = entry.name.toByteArray();
         int var2 = entry.name.size();
         System.arraycopy(var10004, 0, var3, 0, var2);
         var3[entry.name.size()] = 58;
         byte[] var10001 = entry.value.toByteArray();
         Header var10002 = entry;
         int entry1 = entry.name.size() + 1;
         var2 = var10002.value.size();
         System.arraycopy(var10001, 0, var3, entry1, var2);
         return ByteString.of(var10000);
      }

      void writeHeaders(List<Header> headerBlock) {
         int var2;
         if (this.emitDynamicTableSizeUpdate) {
            if ((var2 = this.smallestHeaderTableSizeSetting) < this.maxDynamicTableByteCount) {
               this.writeInt(var2, 31, 32);
            }

            this.emitDynamicTableSizeUpdate = false;
            this.smallestHeaderTableSizeSetting = Integer.MAX_VALUE;
            this.writeInt(this.maxDynamicTableByteCount, 31, 32);
         }

         var2 = 0;

         for(int var3 = headerBlock.size(); var2 < var3; ++var2) {
            Header var4;
            Header var10000 = var4 = (Header)headerBlock.get(var2);
            ByteString var5 = var10000.name.toAsciiLowercase();
            ByteString var6 = var10000.value;
            Integer var7;
            if ((var7 = (Integer)Hpack.NAME_TO_FIRST_INDEX.get(var5)) != null) {
               this.writeInt(var7 + 1, 15, 0);
               this.writeByteString(var6);
            } else {
               ByteString var8 = this.getHeaderString(var4);
               if ((var7 = (Integer)this.headerStringToDynamicIndex.get(var8)) != null) {
                  this.writeInt(this.dynamicTable.length - var7 + Hpack.STATIC_HEADER_TABLE.length, 127, 128);
               } else {
                  this.out.writeByte(64);
                  this.writeByteString(var5);
                  this.writeByteString(var6);
                  this.insertIntoDynamicTable(var4);
               }
            }
         }

      }

      void writeInt(int value, int prefixMask, int bits) {
         if (value < prefixMask) {
            this.out.writeByte(bits | value);
         } else {
            this.out.writeByte(bits | prefixMask);

            int var10000;
            for(value -= prefixMask; value >= 128; value = var10000 >>> 7) {
               var10000 = value;
               value &= 127;
               this.out.writeByte(value | 128);
            }

            this.out.writeByte(value);
         }
      }

      void writeByteString(ByteString data) {
         this.writeInt(data.size(), 127, 0);
         this.out.write(data);
      }

      void setHeaderTableSizeSetting(int headerTableSizeSetting) {
         this.headerTableSizeSetting = headerTableSizeSetting;
         headerTableSizeSetting = Math.min(headerTableSizeSetting, 16384);
         int var2;
         if ((var2 = this.maxDynamicTableByteCount) != headerTableSizeSetting) {
            if (headerTableSizeSetting < var2) {
               this.smallestHeaderTableSizeSetting = Math.min(this.smallestHeaderTableSizeSetting, headerTableSizeSetting);
            }

            this.emitDynamicTableSizeUpdate = true;
            this.maxDynamicTableByteCount = headerTableSizeSetting;
            this.adjustDynamicTableByteCount();
         }
      }
   }

   static final class Reader {
      private final List<Header> headerList;
      private final BufferedSource source;
      private final int headerTableSizeSetting;
      private int maxDynamicTableByteCount;
      Header[] dynamicTable;
      int nextHeaderIndex;
      int headerCount;
      int dynamicTableByteCount;

      Reader(int headerTableSizeSetting, Source source) {
         this(headerTableSizeSetting, headerTableSizeSetting, source);
      }

      Reader(int headerTableSizeSetting, int maxDynamicTableByteCount, Source source) {
         Reader var10000 = this;
         Reader var10002 = this;
         Reader var10004 = this;
         Reader var10006 = this;
         Reader var10007 = this;
         Reader var10008 = this;
         super();
         this.headerList = new ArrayList();
         Header[] var4;
         this.dynamicTable = var4 = new Header[8];
         var10008.nextHeaderIndex = var4.length - 1;
         var10007.headerCount = 0;
         var10006.dynamicTableByteCount = 0;
         var10004.headerTableSizeSetting = headerTableSizeSetting;
         var10002.maxDynamicTableByteCount = maxDynamicTableByteCount;
         var10000.source = Okio.buffer(source);
      }

      private void adjustDynamicTableByteCount() {
         int var1;
         int var2;
         if ((var1 = this.maxDynamicTableByteCount) < (var2 = this.dynamicTableByteCount)) {
            if (var1 == 0) {
               this.clearDynamicTable();
            } else {
               this.evictToRecoverBytes(var2 - var1);
            }
         }

      }

      private void clearDynamicTable() {
         this.headerList.clear();
         Arrays.fill(this.dynamicTable, (Object)null);
         this.nextHeaderIndex = this.dynamicTable.length - 1;
         this.headerCount = 0;
         this.dynamicTableByteCount = 0;
      }

      private int evictToRecoverBytes(int bytesToRecover) {
         int var2 = 0;
         if (bytesToRecover > 0) {
            int var3;
            int var4;
            for(var3 = this.dynamicTable.length - 1; var3 >= (var4 = this.nextHeaderIndex) && bytesToRecover > 0; bytesToRecover = var4) {
               Header[] bytesToRecover1;
               var4 = bytesToRecover - (bytesToRecover1 = this.dynamicTable)[var3].hpackSize;
               this.dynamicTableByteCount -= bytesToRecover1[var3].hpackSize;
               int var10001 = this.headerCount--;
               ++var2;
               --var3;
            }

            Header[] var10002 = this.dynamicTable;
            int var5 = var4 + 1;
            bytesToRecover = var4 + 1 + var2;
            var3 = this.headerCount;
            System.arraycopy(var10002, var5, var10002, bytesToRecover, var3);
            this.nextHeaderIndex += var2;
         }

         return var2;
      }

      private void readIndexedHeader(int index) {
         if (this.isStaticHeader(index)) {
            Reader var10000 = this;
            Header var4 = Hpack.STATIC_HEADER_TABLE[index];
            var10000.headerList.add(var4);
         } else {
            int var2;
            Header[] var3;
            if ((var2 = this.dynamicTableIndex(index - Hpack.STATIC_HEADER_TABLE.length)) < 0 || var2 > (var3 = this.dynamicTable).length - 1) {
               throw new IOException("Header index too large " + (index + 1));
            }

            this.headerList.add(var3[var2]);
         }

      }

      private int dynamicTableIndex(int index) {
         return this.nextHeaderIndex + 1 + index;
      }

      private void readLiteralHeaderWithoutIndexingIndexedName(int index) {
         Reader var10000 = this;
         Reader var10001 = this;
         ByteString var2 = this.getName(index);
         ByteString index1 = var10001.readByteString();
         var10000.headerList.add(new Header(var2, index1));
      }

      private void readLiteralHeaderWithoutIndexingNewName() {
         Reader var10000 = this;
         Reader var10001 = this;
         ByteString var2 = Hpack.checkLowercase(this.readByteString());
         ByteString var1 = var10001.readByteString();
         var10000.headerList.add(new Header(var2, var1));
      }

      private void readLiteralHeaderWithIncrementalIndexingIndexedName(int nameIndex) {
         Reader var10000 = this;
         Reader var10001 = this;
         ByteString var3 = this.getName(nameIndex);
         ByteString nameIndex1 = var10001.readByteString();
         Header var2;
         var2 = new Header.<init>(var3, nameIndex1);
         var10000.insertIntoDynamicTable(-1, var2);
      }

      private void readLiteralHeaderWithIncrementalIndexingNewName() {
         Reader var10000 = this;
         Reader var10001 = this;
         ByteString var3 = Hpack.checkLowercase(this.readByteString());
         ByteString var1 = var10001.readByteString();
         Header var2;
         var2 = new Header.<init>(var3, var1);
         var10000.insertIntoDynamicTable(-1, var2);
      }

      private ByteString getName(int index) {
         return this.isStaticHeader(index) ? Hpack.STATIC_HEADER_TABLE[index].name : this.dynamicTable[this.dynamicTableIndex(index - Hpack.STATIC_HEADER_TABLE.length)].name;
      }

      private boolean isStaticHeader(int index) {
         return index >= 0 && index <= Hpack.STATIC_HEADER_TABLE.length - 1;
      }

      private void insertIntoDynamicTable(int index, Header entry) {
         this.headerList.add(entry);
         int var3 = entry.hpackSize;
         if (index != -1) {
            var3 -= this.dynamicTable[this.dynamicTableIndex(index)].hpackSize;
         }

         int var4;
         if (var3 > (var4 = this.maxDynamicTableByteCount)) {
            this.clearDynamicTable();
         } else {
            var4 = this.evictToRecoverBytes(this.dynamicTableByteCount + var3 - var4);
            if (index == -1) {
               Header[] index1;
               if (this.headerCount + 1 > (index1 = this.dynamicTable).length) {
                  Header[] var10003 = index1;
                  Header[] var10004 = index1;
                  Header[] var10005 = index1;
                  index1 = new Header[index1.length * 2];
                  var4 = var10005.length;
                  int var5 = var10004.length;
                  System.arraycopy(var10003, 0, index1, var4, var5);
                  this.nextHeaderIndex = this.dynamicTable.length - 1;
                  this.dynamicTable = index1;
               }

               this.nextHeaderIndex = (index = this.nextHeaderIndex) - 1;
               this.dynamicTable[index] = entry;
               int var10001 = this.headerCount++;
            } else {
               index += this.dynamicTableIndex(index) + var4;
               this.dynamicTable[index] = entry;
            }

            this.dynamicTableByteCount += var3;
         }
      }

      private int readByte() {
         return this.source.readByte() & 255;
      }

      int maxDynamicTableByteCount() {
         return this.maxDynamicTableByteCount;
      }

      void readHeaders() {
         while(true) {
            if (!this.source.exhausted()) {
               int var1;
               if ((var1 = this.source.readByte() & 255) != 128) {
                  if ((var1 & 128) == 128) {
                     this.readIndexedHeader(this.readInt(var1, 127) - 1);
                     continue;
                  }

                  if (var1 == 64) {
                     this.readLiteralHeaderWithIncrementalIndexingNewName();
                     continue;
                  }

                  if ((var1 & 64) == 64) {
                     this.readLiteralHeaderWithIncrementalIndexingIndexedName(this.readInt(var1, 63) - 1);
                     continue;
                  }

                  if ((var1 & 32) == 32) {
                     int var10000 = var1 = this.readInt(var1, 31);
                     this.maxDynamicTableByteCount = var1;
                     if (var10000 >= 0 && var1 <= this.headerTableSizeSetting) {
                        this.adjustDynamicTableByteCount();
                        continue;
                     }

                     throw new IOException("Invalid dynamic table size update " + this.maxDynamicTableByteCount);
                  }

                  if (var1 != 16 && var1 != 0) {
                     this.readLiteralHeaderWithoutIndexingIndexedName(this.readInt(var1, 15) - 1);
                     continue;
                  }

                  this.readLiteralHeaderWithoutIndexingNewName();
                  continue;
               }

               throw new IOException("index == 0");
            }

            return;
         }
      }

      public List<Header> getAndResetHeaderList() {
         ArrayList var10000 = new ArrayList(this.headerList);
         this.headerList.clear();
         return var10000;
      }

      int readInt(int firstByte, int prefixMask) {
         if ((firstByte &= prefixMask) < prefixMask) {
            return firstByte;
         } else {
            int var3;
            for(firstByte = 0; ((var3 = this.readByte()) & 128) != 0; firstByte += 7) {
               prefixMask += (var3 & 127) << firstByte;
            }

            return prefixMask + (var3 << firstByte);
         }
      }

      ByteString readByteString() {
         int var1;
         boolean var10000 = ((var1 = this.readByte()) & 128) == 128;
         var1 = this.readInt(var1, 127);
         return var10000 ? ByteString.of(Huffman.get().decode(this.source.readByteArray((long)var1))) : this.source.readByteString((long)var1);
      }
   }
}
